SystemOrganization addCategory: #'Mondrian-Events'!

!SubscriptionRegistry methodsFor: '*mondrian-events' stamp: 'AlexandreBergel 5/18/2011 10:02'!
hasHandlerFor: aEventClass
	"true if an handler is associated to the event class"

	^ (subscriptions anySatisfy:  [ :sub | sub announcementClass == aEventClass ])! !

!SubscriptionRegistry methodsFor: '*mondrian-events' stamp: 'AlexandreBergel 5/18/2011 10:00'!
lookFor: aEventClass
	^ self lookFor: aEventClass ifNone: [ ^ nil ]! !

!SubscriptionRegistry methodsFor: '*mondrian-events' stamp: 'AlexandreBergel 5/18/2011 10:01'!
lookFor: aEventClass ifNone: ablock
	"Return the block associated to the event class"

	^ (subscriptions detect: [ :sub | sub announcementClass == aEventClass ] ifNone: [ ^ ablock value]) action! !

!SubscriptionRegistry methodsFor: '*mondrian-events' stamp: 'AlexandreBergel 5/18/2011 10:44'!
unsubscribeForEvent: aEventClass
	| collect |
	collect := subscriptions select: [ :v | v announcementClass == aEventClass ].
	subscriptions := subscriptions copyWithoutAll: collect  ! !

Announcement subclass: #MOEvent
	instanceVariableNames: 'event element current timestamp cancelled viewRenderer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Events'!

MOEvent subclass: #MOContentChanged
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Events'!

!MOContentChanged commentStamp: 'Alexandre.Bergel 3/3/2009 14:05' prior: 0!
Used when the content of a window is changed, and a redraw has to be done!

MOEvent subclass: #MOElementDeselection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Events'!

MOEvent subclass: #MOElementSelection
	instanceVariableNames: 'isMultiple'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Events'!

!MOElementSelection methodsFor: 'initialization' stamp: 'AlexandreBergel 9/5/2010 16:39'!
initializeEvent: aMorphicEvent on: anElement
	super initializeEvent: aMorphicEvent on: anElement.
	isMultiple := false! !

!MOElementSelection methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2010 11:54'!
isMultiple
	^ isMultiple! !

!MOElementSelection methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2010 11:54'!
isMultiple: anObject
	isMultiple := anObject! !

!MOElementSelection methodsFor: 'testing' stamp: 'AlexandreBergel 9/5/2010 16:34'!
isUnique
	^ self isMultiple not! !

!MOElementSelection methodsFor: 'actions' stamp: 'AlexandreBergel 9/5/2010 11:54'!
setAsMultiple 
	isMultiple := true! !

!MOEvent classSide methodsFor: 'instance creation' stamp: 'AlexandreBergel 9/5/2010 16:38'!
event: aMorphicEvent on: anElement
	^ self new initializeEvent: aMorphicEvent on: anElement! !

!MOEvent classSide methodsFor: 'instance creation' stamp: 'AlexandreBergel 10/28/2009 09:57'!
initializedInstance
	"Used to create a ready-to use instance of myself"
	^ self new element: (MONode on: 10); yourself! !

!MOEvent methodsFor: 'actions' stamp: 'lr 10/11/2006 20:52'!
cancel
	cancelled := true! !

!MOEvent methodsFor: 'accessing' stamp: 'Alexandre Bergel 12/17/2009 18:42'!
current
	"Answer the element that is currently dispatching this event."

	^ current! !

!MOEvent methodsFor: 'accessing' stamp: 'Alexandre Bergel 3/2/2010 22:40'!
currentInteraction
	^ self current announcement! !

!MOEvent methodsFor: 'accessing' stamp: 'lr 10/11/2006 20:50'!
element
	"Answer the element that originally received this event."
	
	^ element! !

!MOEvent methodsFor: 'accessing' stamp: 'Alexandre.Bergel 8/30/2009 13:55'!
element: object
	"The element of an announcement may be changed when forwarding. Have a look at sender of isForwarder.
	 MOAnnouncer>>announce:"
	
	element := object! !

!MOEvent methodsFor: 'accessing' stamp: 'Alexandre Bergel 3/2/2010 23:10'!
elementInteraction
	^ self element announcer! !

!MOEvent methodsFor: 'testing' stamp: 'Alexandre.Bergel 8/30/2009 13:36'!
hasElement
	^ element notNil! !

!MOEvent methodsFor: 'initialization' stamp: 'AlexandreBergel 9/5/2010 16:47'!
initialize 
	super initialize.
	self initializeEvent: nil on: nil! !

!MOEvent methodsFor: 'initialization' stamp: 'lr 11/11/2006 19:39'!
initializeEvent: aMorphicEvent on: anElement
	event := aMorphicEvent.
	element := current := anElement.
	timestamp := TimeStamp now.
	cancelled := false! !

!MOEvent methodsFor: 'testing-keyboard' stamp: 'AlexandreBergel 12/23/2010 21:30'!
isAltKeyPressed
	"true if the alt key is pressed"
	^ event commandKeyPressed! !

!MOEvent methodsFor: 'testing' stamp: 'lr 10/11/2006 20:51'!
isCancelled
	^ cancelled! !

!MOEvent methodsFor: 'testing-keyboard' stamp: 'AlexandreBergel 12/23/2010 21:29'!
isCtrlKeyPressed
	"true if the control key is pressed"
	^ event controlKeyPressed! !

!MOEvent methodsFor: 'testing' stamp: 'lr 11/11/2006 00:54'!
isInitiator
	^ element = current! !

!MOEvent methodsFor: 'testing-keyboard' stamp: 'AlexandreBergel 12/23/2010 21:30'!
isShiftKeyPressed
	"true if the shift key is pressed"
	^ event shiftPressed! !

!MOEvent methodsFor: 'accessing' stamp: 'lr 10/18/2006 23:40'!
model
	"Answer the model object that triggered this event."
	
	^ element model! !

!MOEvent methodsFor: 'testing' stamp: 'Alexandre.Bergel 8/30/2009 13:37'!
modelElement
	^ element model! !

!MOEvent methodsFor: 'accessing' stamp: 'Alexandre.Bergel 9/23/2009 17:20'!
node
	^ self element! !

!MOEvent methodsFor: 'printing' stamp: 'lr 10/8/2006 20:48'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' on: '; print: self element! !

!MOEvent methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2010 21:12'!
root
	"Answer the root this event is dispatched in."
	current ifNil: [ ^ element root ].
	^ current root! !

!MOEvent methodsFor: 'morphic' stamp: 'Alexandre Bergel 12/28/2009 11:54'!
setAsNotHandled
	event wasHandled: false! !

!MOEvent methodsFor: 'accessing' stamp: 'lr 10/4/2006 09:49'!
timestamp
	"Answer the timestamp when the event was created."
	
	^ timestamp! !

!MOEvent methodsFor: 'accessing' stamp: 'Alexandre Bergel 12/17/2009 18:55'!
viewRenderer
	^ viewRenderer! !

!MOEvent methodsFor: 'accessing' stamp: 'Alexandre Bergel 12/17/2009 18:55'!
viewRenderer: anObject
	viewRenderer := anObject! !

MOEvent subclass: #MOKeyEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Events'!

MOKeyEvent subclass: #MOKeyDown
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Events'!

!MOKeyEvent methodsFor: 'accessing' stamp: 'lr 10/11/2006 21:19'!
character
	^ event keyCharacter! !

!MOKeyEvent methodsFor: 'printing' stamp: 'lr 10/11/2006 21:06'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' character: '; print: self character! !

MOKeyEvent subclass: #MOKeyPressed
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Events'!

MOKeyEvent subclass: #MOKeyUp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Events'!

MOEvent subclass: #MOMouseEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Events'!

MOMouseEvent subclass: #MOMouseDouble
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Events'!

MOMouseEvent subclass: #MOMouseDown
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Events'!

!MOMouseDown commentStamp: 'Alexandre.Bergel 8/30/2009 14:01' prior: 0!
The mouseDown event occurs when the pointing device button is pressed over an element.!

MOMouseEvent subclass: #MOMouseDrag
	instanceVariableNames: 'stepAsPoint'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Events'!

!MOMouseDrag classSide methodsFor: 'as yet unclassified' stamp: 'Alexandre Bergel 7/13/2010 22:54'!
step: aStep
	^ self new step: aStep! !

!MOMouseDrag methodsFor: 'accessed-computed' stamp: 'AlexandreBergel 9/5/2010 17:06'!
nodesToDrag
	| v |
	v := self element root allSelectedNodes.
	(v includes: self element)
		ifFalse: [ v add: self element ].
	^ v! !

!MOMouseDrag methodsFor: 'accessing' stamp: 'Alexandre.Bergel 2/2/2009 16:02'!
step
	^ stepAsPoint! !

!MOMouseDrag methodsFor: 'accessing' stamp: 'Alexandre.Bergel 2/2/2009 16:02'!
step: aPoint
	stepAsPoint := aPoint! !

!MOMouseDrag methodsFor: 'accessing' stamp: 'Alexandre.Bergel 2/2/2009 15:50'!
stepAsPoint: anObject
	stepAsPoint := anObject! !

MOMouseEvent subclass: #MOMouseDrop
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Events'!

MOMouseEvent subclass: #MOMouseEnter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Events'!

!MOMouseEnter commentStamp: 'Alexandre.Bergel 8/30/2009 14:02' prior: 0!
The mouseEnter event occurs when the pointing device is moved onto an element.!

!MOMouseEvent methodsFor: 'testing-mouse' stamp: 'Alexandre.Bergel 3/31/2009 20:19'!
isLeftButtonPressed
	event ifNil: [ ^ true ]. "This is used in the tests"
	^ event redButtonPressed! !

!MOMouseEvent methodsFor: 'testing-mouse' stamp: 'lr 10/11/2006 21:24'!
isMiddleButtonPressed
	^ event yellowButtonPressed! !

!MOMouseEvent methodsFor: 'testing-mouse' stamp: 'lr 10/11/2006 21:23'!
isRightButtonPressed
	^ event blueButtonPressed! !

!MOMouseEvent methodsFor: 'accessing' stamp: 'Alexandre Bergel 1/26/2010 22:05'!
position
	"Answer the position of the mouse event relative to the root."
	event ifNil: [ ^ 0@0].
	^ event position! !

MOMouseEvent subclass: #MOMouseLeave
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Events'!

!MOMouseLeave commentStamp: 'Alexandre.Bergel 8/30/2009 14:02' prior: 0!
The mouseLeave event occurs when the pointing device is moved away from an element.!

MOMouseEvent subclass: #MOMouseMove
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Events'!

!MOMouseMove commentStamp: 'Alexandre.Bergel 8/30/2009 14:01' prior: 0!
The mouseMove event occurs when the pointing device is moved while it is over an element.!

MOMouseEvent subclass: #MOMouseUp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Events'!

!MOMouseUp commentStamp: 'Alexandre.Bergel 8/30/2009 14:01' prior: 0!
The mouseUp event occurs when the pointing device button is released over an element.!

MOEvent subclass: #MOWindowStatusChanged
	instanceVariableNames: 'newLabel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Events'!

!MOWindowStatusChanged methodsFor: 'accessing' stamp: 'Alexandre Bergel 2/23/2010 23:27'!
newLabel
	^ newLabel! !

!MOWindowStatusChanged methodsFor: 'accessing' stamp: 'Alexandre Bergel 2/23/2010 23:27'!
newLabel: anObject
	newLabel := anObject! !

MOEvent subclass: #MOZoomEvent
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Events'!

!MOZoomEvent classSide methodsFor: 'instance creation' stamp: 'Alexandre.Bergel 3/10/2009 11:46'!
decrease
	^ self basicNew value: #decrease! !

!MOZoomEvent classSide methodsFor: 'instance creation' stamp: 'Alexandre.Bergel 3/10/2009 11:46'!
increase
	^ self basicNew value: #increase! !

!MOZoomEvent classSide methodsFor: 'instance creation' stamp: 'TestRunner 10/28/2009 09:58'!
initializedInstance
	"Used to create a ready-to use instance of myself"
	^ self increase element: (MONode on: 10); yourself! !

!MOZoomEvent classSide methodsFor: 'instance creation' stamp: 'Alexandre.Bergel 3/10/2009 11:45'!
new
	self error: 'please use increase or decrease for instantiating this class'! !

!MOZoomEvent methodsFor: 'testing' stamp: 'Alexandre.Bergel 3/10/2009 11:47'!
isZoomDecrease
	^ value == #decrease! !

!MOZoomEvent methodsFor: 'testing' stamp: 'Alexandre.Bergel 3/10/2009 11:47'!
isZoomIncrease
	^ value == #increase! !

!MOZoomEvent methodsFor: 'accessing' stamp: 'Alexandre.Bergel 3/10/2009 11:47'!
value: aSymbol
	value := aSymbol! !

!MOGraphElement methodsFor: '*Mondrian-Events-testing' stamp: 'Alexandre.Bergel 9/21/2009 23:16'!
isNotSelected
	^ self isSelected not
! !

!MOGraphElement methodsFor: '*Mondrian-Events-testing' stamp: 'tg 9/14/2010 07:05'!
isSelected
	^ isSelected! !

Announcer subclass: #MOAnnouncer
	instanceVariableNames: 'forwards menuItems menuMorphBlock viewRenderer processes'
	classVariableNames: 'RubberBandStartElement'
	poolDictionaries: ''
	category: 'Mondrian-Events'!

!MOAnnouncer methodsFor: 'convenient' stamp: 'Alexandre Bergel 12/28/2009 11:35'!
action: oneArgumentBlockactionOrSymbol

	^ oneArgumentBlockactionOrSymbol isSymbol
		ifTrue: [ self item: oneArgumentBlockactionOrSymbol asString action: oneArgumentBlockactionOrSymbol ]
		ifFalse: [ self item: 'action' action: oneArgumentBlockactionOrSymbol ]
! !

!MOAnnouncer methodsFor: 'announce' stamp: 'AlexandreBergel 9/16/2010 14:48'!
announce: ann
	ann viewRenderer: self viewRenderer.
	(self isForwarder and: [ forwards anySatisfy: [ :ev | ann isKindOf: ev ]]) ifTrue: [ 
			ann element: ann element owner. 
			^ ann element announce: ann ].
	super announce: ann! !

!MOAnnouncer methodsFor: 'convenient' stamp: 'BenComan 5/11/2012 00:25'!
draggable
	self unsubscribeForEvent: MOMouseDrag.
	self subscribe: MOMouseDrag do: [:ann | 
		ann element  isNode ifTrue: [
			ann node isSelected 
				ifTrue: 
					[ ann nodesToDrag do: [ :node |
						node translateBy: ann step bounded: true.
						node resetFormCacheUpToTheRoot ] ]
				ifFalse: 
					[ ann root deselectAll.
					  ann node select.
					  ann node translateBy: ann step bounded: true.
					].
			 ].
		
		"ann element resetFormCacheRecursively."
		ann viewRenderer updateWindow ]
	! !

!MOAnnouncer methodsFor: 'threading' stamp: 'Alexandre Bergel 6/15/2010 09:02'!
every: milliseconds do: block
	| process |
	process := [ [ true ] 
					whileTrue: [ block value": self viewRenderer root" . (Delay forMilliseconds: milliseconds) wait ]] forkNamed: 'mondrian process'.
	self processes add: process ! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'VeronicaUquillas 9/30/2011 15:39'!
forNode: aBlockOrSymbol whenClickingUpdateNode: firstClickBlock whenClickingBackUpdateNode: secondClickBlock withLayoutUpdate: aBoolean
	"enteringBlock and leavingBlock are one arg block each. The argument is the shape of the node that receive the event"

	self registerForEvent: MOMouseDown forNode: aBlockOrSymbol updateNode: firstClickBlock updateLayout: aBoolean.
	self registerForEvent: MOMouseUp forNode: aBlockOrSymbol updateNode: secondClickBlock updateLayout: aBoolean.! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'Alexandre Bergel 6/19/2010 20:16'!
forNode: aBlockOrSymbol whenClickingUpdateNode: firstClickBlock withLayoutUpdate: aBoolean
	"enteringBlock and leavingBlock are one arg block each. The argument is the shape of the node that receive the event"

	self registerForEvent: MOMouseDown forNode: aBlockOrSymbol updateNode: firstClickBlock updateLayout: aBoolean.
! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'Alexandre Bergel 6/19/2010 19:59'!
forNode: aBlockOrSymbol whenEnteringUpdateNode: enteringBlock whenLeavingUpdateNode: leavingBlock withLayoutUpdate: aBoolean

	self registerForEvent: MOMouseEnter forNode: aBlockOrSymbol updateNode: enteringBlock updateLayout: aBoolean.
	self registerForEvent: MOMouseLeave forNode: aBlockOrSymbol updateNode: leavingBlock updateLayout: aBoolean.! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'Alexandre Bergel 1/14/2010 16:40'!
forNodes: aBlockOrSymbol whenEnteringUpdateShapes: enteringBlock whenLeavingUpdateShapes: leavingBlock
	"enteringBlock and leavingBlock are one arg block each. The argument is the shape of the node that receive the event"

	^ self forNodes: aBlockOrSymbol whenEnteringUpdateShapes: enteringBlock whenLeavingUpdateShapes: leavingBlock withLayoutUpdate: false! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'Alexandre Bergel 6/19/2010 20:13'!
forNodes: aBlockOrSymbol whenEnteringUpdateShapes: enteringBlock whenLeavingUpdateShapes: leavingBlock withLayoutUpdate: aBoolean
	"enteringBlock and leavingBlock are one arg block each. The argument is the shape of the node that receive the event"

	self registerForEvent: MOMouseEnter forNodes: aBlockOrSymbol updateShape: enteringBlock updateLayout: aBoolean.
	self registerForEvent: MOMouseLeave forNodes: aBlockOrSymbol updateShape: leavingBlock updateLayout: aBoolean.! !

!MOAnnouncer methodsFor: 'forwarder' stamp: 'AlexandreBergel 9/16/2010 14:46'!
forward
	self forwarder ! !

!MOAnnouncer methodsFor: 'forwarder' stamp: 'AlexandreBergel 12/20/2010 19:37'!
forward: anEventClass
	forwards ifNil: [ forwards := OrderedCollection new ].
	forwards add: anEventClass ! !

!MOAnnouncer methodsFor: 'forwarder' stamp: 'AlexandreBergel 9/16/2010 14:43'!
forwarder
	self forward: MOEvent! !

!MOAnnouncer methodsFor: 'convenient' stamp: 'AlexandreBergel 5/18/2011 10:02'!
hasHandlerFor: aEventClass
	"return the handler associated to the event class"
	^ registry hasHandlerFor: aEventClass! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'VanessaPena 8/19/2011 13:50'!
highlightWhenOver

	| elements domainNodes |
	self 
		registerForEvent: MOMouseEnter 
		forNodes: #yourself 
		updateGraphNode: [ :node | 
			node attributeAt: #oldBorderColor ifAbsentPut: [ node shape borderColor ].
			node copyShapeWithoutUpdatingAndDo: [ :shape | shape borderColor: Color red ] ] 
		updateLayout: false.

	self 
		registerForEvent: MOMouseLeave 
		forNodes: #yourself 
		updateGraphNode: [ :node | 

			node copyShapeWithoutUpdatingAndDo: [ :shape | shape borderColor: (node attributeAt: #oldBorderColor) ] ] 
		updateLayout: false.
! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'AlexandreBergel 5/2/2012 12:08'!
highlightWhenOver: aBlockOrSymbol

	^ self highlightWhenOver: aBlockOrSymbol color: Color red! !

!MOAnnouncer methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 5/2/2012 12:07'!
highlightWhenOver: aBlockOrSymbol color: aColor

	| elements domainNodes |
	self 
		registerForEvent: MOMouseEnter 
		forNodes: aBlockOrSymbol
		updateGraphNode: [ :node | 
			node attributeAt: #oldBorderColor ifAbsentPut: [node shape borderColor].
			node copyShapeWithoutUpdatingAndDo: [ :shape | shape borderColor: aColor ] ] 
		updateLayout: false.

	self 
		registerForEvent: MOMouseLeave 
		forNodes: aBlockOrSymbol
		updateGraphNode: [ :node | 

			node copyShapeWithoutUpdatingAndDo: [ :shape | shape borderColor: (node attributeAt: #oldBorderColor) ] ] 
		updateLayout: false.! !

!MOAnnouncer methodsFor: 'forwarder' stamp: 'AlexandreBergel 9/16/2010 14:45'!
isForwarder
	^ forwards notNil! !

!MOAnnouncer methodsFor: 'convenient' stamp: 'VeronicaUquillas 10/6/2011 12:56'!
item: aStringOrOneArgumentBlockAction action: oneArgumentBlockAction

	self menuItems add: aStringOrOneArgumentBlockAction -> oneArgumentBlockAction.
	self registerMenu! !

!MOAnnouncer methodsFor: 'convenient' stamp: 'AlexandreBergel 5/18/2011 09:46'!
lookFor: aEventClass
	"return the handler associated to the event class"

	^ registry  lookFor: aEventClass

"	^ subscriptions at: aEventClass"! !

!MOAnnouncer methodsFor: 'convenient' stamp: 'tg 9/29/2009 12:05'!
menu: aBlock
	self menuMorphBlock: aBlock! !

!MOAnnouncer methodsFor: 'accessing' stamp: 'tg 9/29/2009 11:13'!
menuItems
	^ menuItems ifNil: [menuItems := OrderedCollection new]! !

!MOAnnouncer methodsFor: 'accessing' stamp: 'tg 9/29/2009 12:05'!
menuMorphBlock: aBlock 
	menuMorphBlock := aBlock.
	self registerMenu	! !

!MOAnnouncer methodsFor: 'convenient' stamp: 'TudorGirba 12/2/2011 12:54'!
menuMorphFor: anElement 
	| menuMorph |
	menuMorph := 
		menuMorphBlock isNil 
			ifTrue:  [ MenuMorph new ]
			ifFalse: [ menuMorphBlock moValue: anElement model ].
		
	self menuItems do: 
		[ :each |
		menuMorph 
			add: (each key moValue: anElement model) 
			target: each value
			selector: #moValue:
			argument: anElement model ].
	^ menuMorph 
	! !

!MOAnnouncer methodsFor: 'convenient' stamp: 'Alexandre.Bergel 4/12/2009 19:45'!
nodraggable
	self unsubscribeForEvent: MOMouseDrag.
! !

!MOAnnouncer methodsFor: 'accessing' stamp: 'Alexandre Bergel 3/15/2010 12:19'!
processes
	^ processes ifNil: [ processes := OrderedCollection new ]! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'Alexandre Bergel 6/15/2010 09:21'!
refreshLayoutEvery: ms
	^ self refreshLayoutEvery: ms view: self viewRenderer ! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'Alexandre Bergel 6/15/2010 09:21'!
refreshLayoutEvery: ms view: view
	self every: ms do: [ 
		view layout doIncrementallyExecute:  view  root.  
		view updateWindow ].! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'AlexandreBergel 4/15/2012 21:18'!
registerForEvent: eventClass forNode: aBlockOrSymbol updateNode: updateBlock updateLayout: aBoolean

	| domainNode |
	self 
		on: eventClass do: [:ann | 
			"ann element = the MONode"
			ann element 
				removeAllEdges;
				removeAllNodes.

			domainNode := (aBlockOrSymbol moValue: ann element model).
			ann element 
				resetMetricCachesRecursively;
				resetElementsToLookupUpToTheRoot.
			
			ann viewRenderer forNodes: {domainNode} do: updateBlock.
			aBoolean
				ifTrue: 
					[ ann viewRenderer root allNodes do: #resetCache.
					  ann viewRenderer root resetElementsToDisplayCacheRecursively.
					  ann element resetCacheInEdges.					
					  ann viewRenderer updateWindow.
					  ann viewRenderer root applyLayout]
				ifFalse: [ ann viewRenderer updateWindow ]
			].
! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'AlexandreBergel 4/15/2012 21:18'!
registerForEvent: eventClass forNode: aBlockOrSymbol updateShape: updateBlock updateLayout: aBoolean

	| node domainNode |
	self 
		on: eventClass do: [:ann | 
			"ann element = the MONode"
			ann element removeAllEdges.

			domainNode := (aBlockOrSymbol moValue: ann element model).
			node := ann viewRenderer nodeForDomainValue: domainNode.
			ann element 
				resetMetricCachesRecursively;
				resetElementsToLookupUpToTheRoot.
			
			node resetMetricCachesRecursively. 
			node copyShapeWithoutUpdatingAndDo: updateBlock.

			aBoolean
				ifTrue: 
					[ ann viewRenderer root allNodes do: #resetCache.
					  ann viewRenderer root resetElementsToDisplayCacheRecursively.
					  ann element resetCacheInEdges.					
					  ann viewRenderer updateWindow.
					  ann viewRenderer root applyLayout]
				ifFalse: [ ann viewRenderer updateWindow ]
			].
! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'AlexandreBergel 4/15/2012 21:18'!
registerForEvent: eventClass forNodes: aBlockOrSymbol updateGraphNode: updateBlock updateLayout: aBoolean

	| domainNodes v |
	self 
		on: eventClass do: [:ann | 
			v := (aBlockOrSymbol moValue: ann element model).
			"v may be a single node or a collection"
			domainNodes := v isCollection ifTrue: [ v ] ifFalse: [ Array with: v ].
			
			
			(ann viewRenderer nodeForDomainValues: domainNodes) do: 
				[ :aNode |
					aNode 
						resetMetricCachesRecursively;
						resetElementsToLookupUpToTheRoot.
					updateBlock value: aNode ].
			
"			ann viewRenderer root allNodes do: #resetCache."
			ann viewRenderer root resetElementsToDisplayCacheRecursively.
			ann element resetCacheInEdges.					
			ann viewRenderer updateWindow.
					
			aBoolean
				ifTrue: 
					[ ann viewRenderer root applyLayout]
			].
! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'AlexandreBergel 4/15/2012 21:18'!
registerForEvent: eventClass forNodes: aBlockOrSymbol updateNode: updateBlock updateLayout: aBoolean

	| domainNodes domainNode |
	self 
		on: eventClass do: [:ann | 
			"ann element = the MONode"
			ann element removeAllEdges.

			domainNodes := (aBlockOrSymbol moValue: ann element model).
			domainNodes do: [:n | 
				n 
					resetMetricCachesRecursively;
					resetElementsToLookupUpToTheRoot ].
			
			ann viewRenderer forNodes: domainNode do: updateBlock.
			aBoolean
				ifTrue: 
					[ ann viewRenderer root allNodes do: #resetCache.
					  ann viewRenderer root resetElementsToDisplayCacheRecursively.
					  ann element resetCacheInEdges.					
					  ann viewRenderer updateWindow.
					  ann viewRenderer root applyLayout]
				ifFalse: [ ann viewRenderer updateWindow ]
			].
! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'AlexandreBergel 4/15/2012 21:18'!
registerForEvent: eventClass forNodes: aBlockOrSymbol updateShape: updateBlock updateLayout: aBoolean

	| nodes domainNodes |
	self 
		on: eventClass do: [:ann | 
			"ann element = the MONode"
			ann element removeAllEdges.

			domainNodes := (aBlockOrSymbol moValue: ann element model).
			nodes := ann viewRenderer nodeForDomainValues: domainNodes.
			ann element 
				resetMetricCachesRecursively;
				resetElementsToLookupUpToTheRoot.
				
			nodes do: [:node |
				node resetMetricCachesRecursively. 
				node copyShapeWithoutUpdatingAndDo: updateBlock ].

			aBoolean
				ifTrue: 
					[ ann viewRenderer root allNodes do: #resetCache.
					  ann viewRenderer root resetElementsToDisplayCacheRecursively.
					  ann element resetCacheInEdges.					
					  ann viewRenderer updateWindow.
					  ann viewRenderer root applyLayout]
				ifFalse: [
					ann element 
						resetFormCacheUpToTheRoot;
						resetMetricCachesUpToTheRoot.
					ann viewRenderer updateWindow ]
			].
! !

!MOAnnouncer methodsFor: 'convenient' stamp: 'Alexandre Bergel 7/19/2010 11:04'!
registerMenu
	self unsubscribeForEvent: MOMouseDown.
	self subscribe: MOMouseDown do: [:ann |
		ann isLeftButtonPressed ifFalse: [
			(self menuMorphFor: ann element) popUpInWorld
		].
		"Clicking on a menu should make popup disappear"
		ann element removePopupIfExist
	].
! !

!MOAnnouncer methodsFor: 'window status' stamp: 'Alexandre Bergel 2/24/2010 00:01'!
resetWindowStatus
	self setWindowStatusTo: '' forElement: viewRenderer root! !

!MOAnnouncer methodsFor: 'convenient' stamp: 'BenComan 6/10/2012 01:54'!
rubberBandFromMe
	"I handle starting and the continued dragging of a rubber band from a node"
	"I never knows about where the moveable end may be dropped. This is handled by rubberBandReceiveDrop."
	"I store the start element in a class variable so that it can be used by rubberBandReceiveDrop"
	"The pixel position of the start and end points of the rubber band is manged by canvas"

	self unsubscribeForEvent: MOMouseDrag.
	self subscribe: MOMouseDrag do: 
	[:ann | |  canvas |
		ann element  isNode ifTrue: 
		[
			RubberBandStartElement := ann element.
			canvas := ann viewRenderer root canvas.
			canvas rubberBand: ann position.
			ann viewRenderer updateWindow 
		]
	].
	! !

!MOAnnouncer methodsFor: 'convenient' stamp: 'BenComan 6/10/2012 02:21'!
rubberBandReceiveDrop: aBlock
	"I take action based on where the moveable end of a rubber band is dropped."
	"The start node is stored for my use in a class variable by #rubberBandFromMe, which I reset."
	"I don't care about what has been happening in the canvas"
	
	| from |
	self unsubscribeForEvent: MOMouseDrop.
	self subscribe: MOMouseDrop do:
	[:ann | 
		"Transcript crShow: 'in edgeDraggedBetweenElements: MOMouseUp' ."
		aBlock value: RubberBandStartElement value: ann element.
		RubberBandStartElement := nil.
	].

	! !

!MOAnnouncer methodsFor: 'window status' stamp: 'tg 4/6/2010 22:50'!
setWindowStatusTo: aString forElement: element
	viewRenderer root announce: ((MOWindowStatusChanged event: nil on: element) newLabel: aString) ! !

!MOAnnouncer methodsFor: 'threading' stamp: 'Alexandre Bergel 3/15/2010 12:47'!
stopProcesses
	processes ifNil: [ ^ self ].
	self processes do: #terminate.
	processes := nil! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'Alexandre Bergel 12/17/2009 19:55'!
strongHighlightWhenOver: aBlockOrSymbol

	self 
		forNodes: aBlockOrSymbol 
		whenEnteringUpdateShapes: [:shape | shape fillColor: Color red ] 
		whenLeavingUpdateShapes: [:shape | shape fillColor: Color white ]! !

!MOAnnouncer methodsFor: 'convenient' stamp: 'AlexandreBergel 5/18/2011 10:04'!
unsubscribeForEvent: aEventClass
	^ registry unsubscribeForEvent: aEventClass! !

!MOAnnouncer methodsFor: 'accessing' stamp: 'Alexandre Bergel 12/17/2009 18:53'!
viewRenderer
	^ viewRenderer! !

!MOAnnouncer methodsFor: 'accessing' stamp: 'Alexandre Bergel 12/17/2009 18:53'!
viewRenderer: anObject
	viewRenderer := anObject! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'Alexandre Bergel 6/19/2010 19:40'!
whenClickingUpdateNode: enteringBlock 
	^ self whenClickingUpdateNode: enteringBlock withLayoutUpdate: false! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'VeronicaUquillas 9/30/2011 15:42'!
whenClickingUpdateNode: enteringBlock whenClickingBackUpdateNode: leavingBlock withLayoutUpdate: aBoolean
	^ self forNode: #yourself whenClickingUpdateNode: enteringBlock whenClickingBackUpdateNode: leavingBlock withLayoutUpdate: aBoolean! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'Alexandre Bergel 6/19/2010 19:41'!
whenClickingUpdateNode: enteringBlock whenLeavingUpdateNode: leavingBlock withLayoutUpdate: aBoolean
	^ self forNode: #yourself whenClickingUpdateNode: enteringBlock  withLayoutUpdate: aBoolean! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'Alexandre Bergel 6/19/2010 20:10'!
whenClickingUpdateNode: clickBlock withLayoutUpdate: aBoolean
	^ self registerForEvent: MOMouseDown forNode: #yourself updateNode: clickBlock updateLayout: aBoolean! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'Alexandre Bergel 6/19/2010 20:06'!
whenClickingUpdateShape: enteringBlock 
	^ self registerForEvent: MOMouseDown forNode: #yourself updateShape: enteringBlock updateLayout: false! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'Alexandre Bergel 6/19/2010 20:06'!
whenClickingUpdateShape: enteringBlock updateLayout: aBoolean
	^ self registerForEvent: MOMouseDown forNode: #yourself updateShape: enteringBlock updateLayout: aBoolean! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'Alexandre Bergel 6/10/2010 18:29'!
whenEnteringUpdateNode: enteringBlock 
	^ self whenEnteringUpdateNode: enteringBlock withLayoutUpdate: false! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'Alexandre Bergel 1/12/2010 14:19'!
whenEnteringUpdateNode: enteringBlock whenLeavingUpdateNode: leavingBlock withLayoutUpdate: aBoolean
	^ self forNode: #yourself whenEnteringUpdateNode: enteringBlock whenLeavingUpdateNode: leavingBlock withLayoutUpdate: aBoolean! !

!MOAnnouncer methodsFor: 'convenient interaction' stamp: 'Alexandre Bergel 4/26/2010 10:53'!
whenEnteringUpdateNode: enteringBlock withLayoutUpdate: aBoolean
	^ self whenEnteringUpdateNode: enteringBlock whenLeavingUpdateNode: [:each | ] withLayoutUpdate: aBoolean! !

!MOAnnouncer methodsFor: 'window status' stamp: 'Alexandre Bergel 2/23/2010 22:31'!
windowStatus: aBlock
	^ self windowStatus: aBlock delay: 500! !

!MOAnnouncer methodsFor: 'window status' stamp: 'AlexandreBergel 9/6/2010 18:34'!
windowStatus: aBlock delay: ms

	"change the window status when the mouse enter a node"
	self unsubscribeForEvent: MOMouseEnter.
	self unsubscribeForEvent: MOMouseLeave.

	self subscribe: MOMouseEnter do: 
		[:ann | (ann hasElement and: [ ann element isNotRoot ])
				ifTrue: [ [ | oldPosition relativePosition |
							oldPosition := ActiveHand position.
					 		(Delay forMilliseconds: ms) wait.
					
					 		"we compute the relative position since ActiveHand return an absolute location"
					 		relativePosition := ann position - oldPosition + ActiveHand position.
					
					 		"We check if we are in the same node, if yes, then we evaluate the view"
					 		(ann root notNil and: [(ann root elementAt: relativePosition) == ann current]) ifTrue: 
								[ self setWindowStatusTo: ([aBlock value: ann modelElement ] on: Error do: ['Error when printing']) printString 
										forElement: ann element]] fork ]
				ifFalse: [ self resetWindowStatus ]].

	self subscribe: MOMouseLeave do: 
		[:ann | self resetWindowStatus ].
! !
